<?php
namespace addons\ygpay\controller;


use addons\ygpay\library\QRCode;
use addons\ygpay\library\Service;
use think\addons\Controller;
use think\Exception;
use think\Response;
use YunGouOS\order\Order;
use YunGouOS\util\PaySign;
use YunGouOS\wxpay\WxPay;

class Api extends Controller
{
    protected $layout = 'default';
    protected $config = [];

    public function index()
    {
        return;
    }

    /**
     * 外部提交
     */
    public function submit()
    {
        $this->request->filter('trim');
        $out_trade_no = $this->request->request("out_trade_no");
        $title = $this->request->request("title");
        $amount = $this->request->request('amount');
        $type = $this->request->request('type');
        $method = $this->request->request('method', 'web');
        $openid = $this->request->request('openid', '');
        $auth_code = $this->request->request('auth_code', '');
        $notifyurl = $this->request->request('notifyurl', '');
        $returnurl = $this->request->request('returnurl', '');

        if (!$amount || $amount < 0) {
            $this->error("支付金额必须大于0");
        }

        if (!$type || !in_array($type, ['alipay', 'wechat'])) {
            $this->error("支付类型错误");
        }

        $params = [
            'type'         => $type,
            'out_trade_no' => $out_trade_no,
            'title'        => $title,
            'amount'       => $amount,
            'method'       => $method,
            'openid'       => $openid,
            'auth_code'    => $auth_code,
            'notifyurl'    => $notifyurl,
            'returnurl'    => $returnurl,
        ];
        return Service::submitOrder($params);
    }

    /**
     * 微信支付(公众号支付&PC扫码支付)
     * @return string
     */
    public function wechat()
    {
        $config = Service::getConfig('wechat');

        $isWechat = stripos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMobile = $this->request->isMobile();
        $this->view->assign("isWechat", $isWechat);
        $this->view->assign("isMobile", $isMobile);

        if($isWechat){
            //微信公众号jsapi支付
        }else{
            //发起PC支付(Scan支付)(PC扫码模式)
            $body = $this->request->request("body");
            $code_url = $this->request->request("code_url");
            $out_trade_no = $this->request->request("out_trade_no");
            $return_url = $this->request->request("return_url");
            $total_fee = $this->request->request("total_fee");
            $sign = $this->request->request("sign");

            $data = [
                'body'         => $body,
                'code_url'     => $code_url,
                'out_trade_no' => $out_trade_no,
                'return_url'   => $return_url,
                'total_fee'    => $total_fee,
            ];
            if ($sign != md5(implode('', $data) . $config['wechat']['key'])) {
                $this->error("签名不正确");
            }

            if ($this->request->isAjax()) {//订单查询
                $key = $config["wechat"]['key'];
                $mch_id = $config["wechat"]['mch_id'];
                $order = new Order();
                try{
                    $result=$order->getOrderInfoByOutTradeNo($out_trade_no,$mch_id,$key);
                }catch (Exception $e){
                    $this->error("查询失败:".$e->getMessage());
                }

                if ($result['payStatus'] == 1) {
                    $this->success("", "", ['trade_state' => "SUCCESS"]);
                } else {
                    $this->error("查询失败");
                }
            }
            $data['sign'] = $sign;
            $type = 'pc';
            $this->view->assign("data", $data);
        }

        $this->view->assign("type", $type);
        $this->view->assign("title", "微信支付");
        return $this->view->fetch();
    }

    /**
     * 支付宝支付(PC扫码支付)
     * @return string
     */
    public function alipay()
    {

    }

    /**
     * 支付成功回调
     */
    public function notifyx()
    {
        $type = $this->request->param('type');
        $data = $this->request->request('', null, 'trim');
        $config = Service::getConfig($type);
        $key=$config[$type]['key'];
        $paySign = new PaySign();
        $result=$paySign->checkNotifySign($data,$key);
        if(!$result){
            echo '签名错误';
            return;
        }


        //你可以在这里你的业务处理逻辑,比如处理你的订单状态、给会员加余额等等功能
        //下面这句必须要执行,且在此之前不能有任何输出
        echo "SUCCESS";
        return;
    }

    /**
     * 支付成功返回
     */
    public function returnx()
    {
        //你可以在这里定义你的提示信息,但切记不可在此编写逻辑
        //$this->success("恭喜你！支付成功!", addon_url("ygpay/index/index"));
        $this->redirect(addon_url("ygpay/index/index"));
        return;
    }

    /**
     * 生成二维码
     */
    public function qrcode()
    {
        $text = $this->request->get('text', 'hello world');

        //如果有安装二维码插件，则调用插件的生成方法
        if (class_exists("\addons\qrcode\library\Service") && get_addon_info('qrcode')['state']) {
            $qrCode = \addons\qrcode\library\Service::qrcode(['text' => $text]);
            $response = Response::create()->header("Content-Type", "image/png");

            header('Content-Type: ' . $qrCode->getContentType());
            $response->content($qrCode->writeString());
            return $response;
        } else {
            $qr = QRCode::getMinimumQRCode($text);
            $im = $qr->createImage(8, 5);
            header("Content-type: image/png");
            imagepng($im);
            imagedestroy($im);
            return;
        }
    }

    /**
     * ask问答插件 企业支付通知和回调
     * @throws \think\exception\DbException
     */
    public function ask()
    {
        $type = $this->request->param('type');
        $paytype = $this->request->param('paytype');
        if ($type == 'notify') {
            $data = $this->request->request('', null, 'trim');
            $config = Service::getConfig($paytype);
            $key=$config[$paytype]['key'];
            $paySign = new PaySign();

            $result=$paySign->checkNotifySign($data,$key);
            if(!$result){
                echo '签名错误';
                return;
            }
            try {
                \addons\ask\library\Order::settle($data['outTradeNo'], $data['money']);
            } catch (Exception $e) {
            }
            echo 'SUCCESS';
            return;
        } else {
            $this->success("支付完成,请关闭当前页面", "/");
        }
        return;
    }
}