<?php
namespace addons\ygpay\library;

use think\Exception;
use think\Response;
use YunGouOS\wxpay\WxPay;

/**
 * 下单服务类
 * Class Service
 * @package addons\ygpay\library
 */
class Service
{
    /**
     * @param $amount
     * @param null $orderid
     * @param null $type
     * @param null $title
     * @param null $notifyurl
     * @param null $returnurl
     * @param null $method
     * @param string $openid
     * @return false|string|null
     * @throws Exception
     */
    public static function submitOrder($amount, $orderid = null, $type = null, $title = null, $notifyurl = null, $returnurl = null, $method = null, $openid = '')
    {
        if (!is_array($amount)) {
            $params = [
                'amount'    => $amount,
                'orderid'   => $orderid,
                'type'      => $type,
                'title'     => $title,
                'notifyurl' => $notifyurl,
                'returnurl' => $returnurl,
                'method'    => $method,
                'openid'    => $openid,
            ];
        } else {
            $params = $amount;
        }
        $type = isset($params['type']) && in_array($params['type'], ['alipay', 'wechat']) ? $params['type'] : 'wechat';
        $method = isset($params['method']) ? $params['method'] : 'web';
        $orderid = isset($params['orderid']) ? $params['orderid'] : date("YmdHis") . mt_rand(100000, 999999);
        $amount = isset($params['amount']) ? $params['amount'] : 1;
        $title = isset($params['title']) ? $params['title'] : "支付";
        $auth_code = isset($params['auth_code']) ? $params['auth_code'] : '';
        $openid = isset($params['openid']) ? $params['openid'] : '';

        $request = request();
        $config = Service::getConfig($type);

        if(isset($params['notifyurl'])){
            $config[$type]['notify_url'] = $params['notifyurl'];
        }
        if(isset($params['returnurl'])){
            $config[$type]['return_url'] = $params['returnurl'];
        }

        $isWechat = strpos($request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;

        $result = null;
        if ($type == 'alipay') {
            exit('暂不支持支付宝');
        } else {
            //如果是PC支付,判断当前环境,进行跳转
            if ($method == 'web') {
                //如果是移动端，但不是微信环境
                if ($request->isMobile() && !$isWechat) {
                    $method = 'wap';
                }
            }

            $wxpay = new WxPay();
            $mch_id = $config[$type]['mch_id'];
            $key = $config[$type]['key'];
            $notifyurl = $config[$type]['notify_url'];
            $returnurl = $config[$type]['return_url'];

            switch ($method) {
                case 'web':
                case 'scan':
                    $result = $wxpay->nativePay($orderid, $amount, $mch_id, $title, '1', null, $notifyurl, null, null, null, $key);
                    $params = [
                        'body' => $title,
                        'code_url' => $result,
                        'out_trade_no' => $orderid,
                        'return_url' => $returnurl,
                        'total_fee' => $amount,
                    ];
                    $params['sign'] = md5(implode('', $params) . $key);
                    $endpoint = addon_url("ygpay/api/wechat");
                    $html = Service::buildPayHtml($endpoint, $params);
                    Response::create($html)->send();
                    break;
                case 'wap':
                case 'app':
                    //个人资质无法开通H5支付权限 如果使用WAP访问 走收银台接口
                    //收银台返回的是收银台的支付连接，此处直接重定向到收银台即可
                    $result = $wxpay->cashierPay($orderid, $amount, $mch_id, $title, null, $notifyurl, $returnurl, null, null, null, $key);
                    header("Location: ".$result."");exit;
                    break;
                case 'mp':
                    //公众号支付
                    //公众号支付必须有openid
                    $result = $wxpay->jsapiPay($orderid, $amount, $mch_id, $title, $openid, null, $notifyurl, null, null, null, $key);
                    break;
                case 'pos':
                    //刷卡支付,直接返回字符串
                    //刷卡支付必须要有auth_code
                    $result = $wxpay->codePay($orderid, $amount, $mch_id, $title, $auth_code, null, 0, $notifyurl, null, null, null, $key);
                    break;
                case 'miniapp':
                    //小程序支付 小程序端参考：https://open.pay.yungouos.com/#/api/api/pay/wxpay/minPay 集成
                    //微信政策原因，不需要走后端请求，小程序段跳转到“支付收银” 支付收银完成了后台API对接
                    return '';
                    break;
                default:
            }
        }

        $result = is_array($result) ? json_encode($result) : $result;
        return $result;
    }

    public static function getConfig($type = 'wechat')
    {
        $config = get_addon_config('ygpay');
        $config = isset($config[$type]) ? $config[$type] : $config['wechat'];
        if ($config['log']) {
            $config['log'] = [
                'file'  => LOG_PATH . 'ygpaylogs' . DS . $type . '-' . date("Y-m-d") . '.log',
                'level' => 'debug'
            ];
        }
        if (isset($config['cert_client']) && substr($config['cert_client'], 0, 8) == '/addons/') {
            $config['cert_client'] = ROOT_PATH . str_replace('/', DS, substr($config['cert_client'], 1));
        }
        if (isset($config['cert_key']) && substr($config['cert_key'], 0, 8) == '/addons/') {
            $config['cert_key'] = ROOT_PATH . str_replace('/', DS, substr($config['cert_key'], 1));
        }
        // 可选
        $config['http'] = [
            'timeout'         => 10,
            'connect_timeout' => 10,
            // 更多配置项请参考 [Guzzle](https://guzzle-cn.readthedocs.io/zh_CN/latest/request-options.html)
        ];

        $config['notify_url'] = empty($config['notify_url']) ? addon_url('ygpay/api/notifyx', [], false) . '/type/' . $type : $config['notify_url'];
        $config['notify_url'] = !preg_match("/^(http:\/\/|https:\/\/)/i", $config['notify_url']) ? request()->root(true) . $config['notify_url'] : $config['notify_url'];
        $config['return_url'] = empty($config['return_url']) ? addon_url('ygpay/api/returnx', [], false) . '/type/' . $type : $config['return_url'];
        $config['return_url'] = !preg_match("/^(http:\/\/|https:\/\/)/i", $config['return_url']) ? request()->root(true) . $config['return_url'] : $config['return_url'];
        return [$type => $config];
    }

    /**
     * 构建支付hmtl
     */
    public static function buildPayHtml($endpoint, $params)
    {
        $sHtml = "<form id='alipaysubmit' name='wechatsubmit' action='" . $endpoint . "' method='POST'>";
        foreach ($params as $key => $val) {
            $val = str_replace("'", '&apos;', $val);
            $sHtml .= "<input type='hidden' name='" . $key . "' value='" . $val . "'/>";
        }
        $sHtml .= "<input type='submit' value='ok' style='display:none;'></form>";
        $sHtml .= "<script>document.forms['wechatsubmit'].submit();</script>";

        return $sHtml;
    }
}